import math

import pygame
from Mind import Orientation, Existence

from . import effects

class recorder:
    def __init__(self):
        self.count = {}
        self.recorded = globals()
        self.pl_is_dead = False

    def add_obj(self, name, obj):
        num = self.count.get(name, 1)
        self.count[name] = num + 1
        self.recorded[name + str(num)] = obj

    def get_prop(self, string):
        return eval(string, self.recorded)


@Existence.logic_class
class platform:
    def __init__(self):
        self.picture = pygame.Surface((self.width, self.height))
        self.picture.fill((0, 255, 0))
        self.fx = 0
        self.fy = 0
        self.stx = self.x
        self.sty = self.y
        self.groups[0].funcs = []

    def blit(self):
        for prop in self.props:
            setattr(self, prop, self.rec.get_prop(self.props[prop]))
        self.set_position(self.stx, self.sty)
        self.move(self.fx, self.fy)
        self.stx = self.x
        self.sty = self.y

    def send(self, keyboard, images, rec):
        self.width = self.obj.width
        self.height = self.obj.height
        self.rec = rec
        self.rec.add_obj("platform", self)


@Existence.logic_class
class player:
    def __init__(self):
        self.fx = 0
        self.fy = 0
        self.ground = False
        self.groups[0].funcs = []
        self.cl_objects = []
        self.M_w = self.Map.width + 100
        self.M_h = self.Map.height + 100
        self.plat = None
        self.stx = self.x
        self.sty = self.y

    def blit(self):
        self.picture = self.pictures[self.fx < 0]
        if self.ground:
            self.fx *= 0.95
        else:
            self.fx *= 0.99
        self.fy += 0.05
        if self.keyboard["up"] == 1 and self.ground:
            self.fy -= 3
        if self.keyboard["left"]:
            self.fx -= 0.1
        if self.keyboard["right"]:
            self.fx += 0.1
        if self.x < -100 or self.x > self.M_w or self.y < -100 or self.y > self.M_h:
            self.rec.pl_is_dead = True
        for prop in self.props:
            setattr(self, prop, self.rec.get_prop(self.props[prop]))
        self.ground = False
        self.plat = None
        self.set_position(self.stx, self.sty)
        self.move(self.fx, self.fy)
        self.stx = self.x
        self.sty = self.y

    def send(self, keyboard, images, rec):
        self.keyboard = keyboard
        self.pictures = images[self.type]
        self.width, self.height = self.pictures[0].get_size()
        self.picture = self.pictures[0]
        self.obj = Orientation.rect(self.x, self.y, self.width, self.height, self.Map.in_map)
        self.rec = rec
        self.rec.add_obj("player", self)

    def left(self, amount, obj):
        self.set_position(obj.x + obj.width, self.y)
        self.fx = 0

    def down(self, amount, obj):
        self.set_position(self.x, obj.y - self.height)
        self.ground = True
        self.plat = obj
        self.fy = 0

    def right(self, amount, obj):
        self.set_position(obj.x - self.width, self.y)
        self.fx = 0

    def up(self, amount, obj):
        self.set_position(self.x, obj.y + obj.height)
        self.fy = 0

    coll = [right, down, left, up, None]


@Existence.logic_class
class end:
    def blit(self):
        self.move(0, 0)

    def send(self, keyboard, images, rec):
        self.picture = images["portal"]
        self.width, self.height = self.picture.get_size()
        self.obj = Orientation.rect(self.x, self.y, self.width, self.height, self.Map.in_map)


class level:
    sender = Existence.group(part=True)
    sender.add_func("send")
    NEXTLEV = [None] * 5
    decode = {
        "player": Existence.mov_type(None, None, sender,
          logic=Existence.Subject + Existence.collider(player.cls.coll,
          "platform") + player),
        "AI": Existence.mov_type(None, None, sender,
          logic=Existence.collider(player.cls.coll, "platform", "player") + player),
        "platform": Existence.mov_type(None, None, sender, logic=platform),
        "nextlev": Existence.mov_type(None, None, sender,
          logic=Existence.collider(NEXTLEV, "player", "AI") + end)
    }
    def __init__(self, num, places, keyboard, **useless):
        self.sender.objects = []
        self.sender.funcs = []

        self.Map = Orientation.tiled_map("art/levels/" + str(num), [{}, {}, self.decode])
        self.props = {}
        for prop in self.Map.out_map.find("properties"):
            self.props[prop.attrib["name"]] = prop.attrib["value"]

        self.message = effects.text(self.props["text"])

        self.recorder = recorder()
        self.places = [places]
        self.keyboard = keyboard
        pictures = {
            "player": [pygame.image.load("art/images/player/1.bmp"), pygame.image.load("art/images/player/2.bmp")],
            "AI": [pygame.image.load("art/images/AI/1.bmp"), pygame.image.load("art/images/AI/2.bmp")],
            "portal": pygame.image.load("art/images/end.bmp")
        }
        self.sender.send(self.keyboard, pictures, self.recorder)
        self.STATE = None
        self.NEXTLEV[-1] = self.next_level

    def blit(self):
        self.keyboard.update()

        self.Map.blit()

        if self.recorder.pl_is_dead:
            self.STATE = "DEAD"

        self.message.blit()

    def set_game(self, game):
        self.game = game

    def next_level(self, *args):
        self.STATE = "NEXT"
